package membership;

import java.util.*;
import io.*;


/**
 * An encapsulation of a membership account at Big Box Bargains.
 * 
 * @author CS159 Professors, James Madison University
 * @version 1
 */
public class Account implements CSVRepresentable {

    private double creditsUsed, purchases;


    /**
     * Default Constructor.
     * 
     * Constructs an Account with 0 purchases and 0 credits used.
     */
    public Account() {
        this(0.00, 0.00);
    }

    /**
     * Explicit Value Constructor.
     * 
     * @param purchases  The dollar amount of purchases
     * @param creditsUsed The dollar amount of credits used
     */
    public Account(double purchases, double creditsUsed) {
        this.creditsUsed = creditsUsed;
        this.purchases = purchases;
    }

    /**
     * Get the available credit (which is based on a percentage of the
     * purchases).
     * 
     * @return The available (i.e., unused) credit
     */
    public double availableCredit() {
        return Math.max(purchases*getRewardPercentage() - creditsUsed, 0.00);
    }

    /**
     * Returns true if this Account entitles the member to use
     * the express line at the store.
     * 
     * @return true if entitled; false otherwise
     */
    public boolean canUseExpressLine() {
        return getPurchases() > 1000.00;
    }

    /**
     * Get the dollar amount of purchases made by the Account holder.
     * 
     * @return The purchases
     */
    public double getPurchases() {
        return purchases;
    }

    /**
     * Get the dollar amount of credits used by the Account holder.
     * 
     * @return The credits used
     */
    public double getCreditsUsed() {
        return creditsUsed;
    }


    /**
     * Get the reward percentage (i.e., the percentage of the Account holder's
     * purchases that they are credited with).
     * 
     * @return The reward percentage
     */
    public double getRewardPercentage() {
        return 0.01;
    }

    /**
     * Increase the credits used by the given amount.
     * 
     * @param credit The amount of the increase
     */
    protected void increaseCreditsUsed(double credit) {
        creditsUsed += Math.max(credit, 0);
    }

    /**
     * Increase the purchases by the given amount.
     * 
     * @param amount The amount of the increase
     */
    protected void increasePurchases(double amount) {
        purchases += Math.max(amount, 0);
    }

    /**
     * Process a purchase.
     * 
     * This method updates the total dollar amount of purchases made by the
     * Account holder and, as appropriate the total dollar amount of credits
     * used by this Account holder.
     * 
     * @param amount The dollar amount of the purchase
     * @param applyCredits true to apply all of the available credits (up to the
     * purchase amount)
     * @return The amount due for this purchase (i.e., the amount minus the
     * credits used)
     */
    public double purchase(double amount, boolean applyCredits) {
        if (amount <= 0.00) return 0.00;

        double credit, result;

        credit = 0.00;
        result = amount;
        if (applyCredits) {
            credit = Math.min(amount, availableCredit());
            result = amount - credit;
        }

        increaseCreditsUsed(credit);
        increasePurchases(result);

        return result;
    }

    /**
     * Populate the attributes of this Account from a CSV representation.
     * 
     * @param s The CSV representation
     * @return The object that was used to tokenize the CSV representation
     */
    @Override
    public StringTokenizer fromCSV(String s) {
        StringTokenizer st;

        st = new StringTokenizer(s, ",");
        this.purchases = Double.parseDouble(st.nextToken());
        this.creditsUsed = Double.parseDouble(st.nextToken());

        // They can have it return a StringTokenizer or a Scanner
        return st;
    }

    /**
     * Create a CSV representation of this Account object.
     * 
     * @return The CSV representation
     */
    @Override
    public String toCSV() {
        return String.format("%.2f,%.2f", getPurchases(), getCreditsUsed()); 
    }

    /**
     * Create a String representation of this Account object.
     * 
     * @return The String representation
     */
    @Override
    public String toString() {
        String fs = "Purchases: %.2f\nCredits Used: %.2f\n"
                + "Credits available: %.2f";
        return String.format(fs, getPurchases(), getCreditsUsed(),
                availableCredit());
    }
}
